@@ -3,10 +3,10 @@
 #' @keywords internal
 #'
 #' @importFrom ggplot2 ggplot ggsave geom_area geom_line geom_vline aes labs %+% element_text geom_point guide_legend scale_colour_manual scale_x_continuous scale_y_continuous theme geom_segment geom_text unit arrow guides theme_light
-#' @importFrom stats loess predict
-#' @importFrom dplyr %>% arrange mutate select filter group_by bind_rows if_else left_join pull slice summarise ungroup
+#' @importFrom stats smooth.spline predict median loess
+#' @importFrom dplyr %>% arrange mutate select filter group_by bind_rows if_else left_join inner_join pull slice summarise ungroup
 #' @importFrom tidyr spread
-#' @importFrom utils tail
+#' @importFrom utils tail head
 transformseries.multiple <- function(i.data,
                                      i.max.epidemic.duration = NA,
                                      i.max.season.duration = NA,
@@ -16,7 +16,7 @@ transformseries.multiple <- function(i.data,
                                      i.min.separation = 1,
                                      i.output = NA,
                                      i.prefix = "Multiple waves",
-                                     i.force.loess = F,
+                                     i.force.smooth = F,
                                      ...) {
   p1 <- list()
   p2 <- list()
@@ -60,43 +60,70 @@ transformseries.multiple <- function(i.data,
     max.waves <- 2 * NCOL(i.data)
   }
   # Prepare the data
-  yrweek <- season <- year <- week <- NULL
-  data <- transformdata.back(i.data)$data %>%
+  # data <- transformdata.back(i.data)$data %>%
+  #   dplyr::arrange(yrweek) %>%
+  #   dplyr::mutate(n = 1:n()) %>%
+  #   dplyr::select(-season, -year, -week)
+  # temp1 <- data %>%
+  #   filter(!is.na(rates))
+  # model.smooth <- smooth.spline(temp1$n, temp1$rates)
+  # p.model.smooth <- predict(model.smooth, x =  data$n)$y
+  # temp1 contains the median of the values outside the epidemic, I take only the 1/3 lowest rates.
+  temp1 <- i.data %>%
+    as.matrix() %>%
+    as.numeric() %>%
+    sort() %>%
+    head(NCOL(i.data) * NROW(i.data) / 3) %>%
+    median()
+  yrweek <- season <- year <- week <- rates <- mrate <- mratej <- NULL
+  # I create the dataset for analysis, i fill missing values outside the epidemic period with the
+  # values I've just calculated before (median values outside epidemic) with a little jitter
+  data <- i.data %>%
+    apply(2, fill.missing) %>%
+    as.data.frame() %>%
+    transformdata.back(i.range.x.final = c(1, 53)) %>%
+    `[[`(1) %>%
     dplyr::arrange(yrweek) %>%
     dplyr::mutate(n = 1:n()) %>%
-    dplyr::select(-season, -year, -week)
-  model.loess <- loess(rates ~ n, data, span = 0.05)
-  rates <- rates.loess <- NULL
+    dplyr::select(-season, -year, -week) %>%
+    mutate(mrate = temp1, mratej = jitter(mrate), y = ifelse(is.na(rates), mratej, rates)) %>%
+    select(-mrate, -mratej)
+  model.smooth <- loess(y ~ n, data, span = 0.05)
+  p.model.smooth <- predict(model.smooth, newdata = data$n)
+  p.model.smooth[p.model.smooth < 0] <- 0
+  rm(temp1)
+  rates <- rates.smooth <- NULL
   data.plus <- data %>%
-    dplyr::mutate(
-      rates.orig = rates,
-      rates.loess = predict(model.loess, newdata = data$n),
-      rates.filled = if_else(is.na(rates), rates.loess, rates)
-    ) %>%
+    dplyr::select(-y) %>%
+    dplyr::mutate(rates.orig = rates)
+  data.plus$rates.smooth <- p.model.smooth
+  # In data.plus I've got the original data, the smoothed data and the filled data (filling missing
+  # values with that of the prediction)
+  data.plus <- data.plus %>%
+    dplyr::mutate(rates.filled = if_else(is.na(rates), rates.smooth, rates)) %>%
     dplyr::select(-rates) %>%
     dplyr::arrange(n)
-  if (i.force.loess) data.plus$rates.filled <- data.plus$rates.loess
-  # Plot the original series, the loess and the data that will be used
-
+  if (i.force.smooth) data.plus$rates.filled <- data.plus$rates.smooth
+  # Plot the original series, the smooth and the data that will be used
   axis.x.range.original <- range(data.plus$n, na.rm = T)
   axis.x.otick <- optimal.tickmarks(axis.x.range.original[1], axis.x.range.original[2], 10, i.include.min = T, i.include.max = T)
   axis.x.range <- axis.x.otick$range
   axis.x.ticks <- axis.x.otick$tickmarks
   axis.x.labels <- data.plus$yrweek[axis.x.otick$tickmarks]
-  axis.y.range.original <- range(c(data.plus$rates.orig, data.plus$rates.loess), na.rm = T)
+  axis.y.range.original <- range(c(data.plus$rates.orig, data.plus$rates.smooth), na.rm = T)
   axis.y.otick <- optimal.tickmarks(axis.y.range.original[1], axis.y.range.original[2], 10)
   axis.y.range <- axis.y.otick$range + diff(range(axis.y.otick$range, na.rm = T)) * 0.025 * c(-1, 1)
   axis.y.ticks <- axis.y.otick$tickmarks
   axis.y.labels <- axis.y.otick$tickmarks
-  n <- rates.orig <- rates.loess <- NULL
+  n <- rates.orig <- rates.smooth <- NULL
   p1[[1]] <- ggplot(data.plus) +
     geom_line(aes(x = n, y = rates.orig), color = "#0066CC", linetype = 1, size = 0.75) +
     geom_point(aes(x = n, y = rates.orig), color = "#0066CC", size = 1.5) +
-    geom_line(aes(x = n, y = rates.loess), color = "#CC0066", linetype = 1, size = 0.75) +
-    geom_point(aes(x = n, y = rates.loess), color = "#CC0066", size = 1.5) +
+    geom_line(aes(x = n, y = rates.smooth), color = "#CC0066", linetype = 1, size = 0.75) +
+    geom_point(aes(x = n, y = rates.smooth), color = "#CC0066", size = 1.5) +
     scale_x_continuous(breaks = axis.x.ticks, limits = axis.x.range, labels = axis.x.labels) +
     scale_y_continuous(breaks = axis.y.ticks, limits = axis.y.range, labels = axis.y.labels) +
-    labs(title = "Series and loess", x = "Week", y = "Data") +
+    labs(title = "Series and smooth", x = "Week", y = "Data") +
     theme_light() +
     theme(plot.title = element_text(hjust = 0.5))
   p1[[2]] <- ggplot(data.plus) +
@@ -116,29 +143,53 @@ transformseries.multiple <- function(i.data,
     names(peradd) <- c("percentage", "start", "end", "duration", "sum", "max")
     n.chosen <- head((1:max.epidemic.duration)[peradd$percentage < (param.1 / 100)], 1) - 1
     peradd.chosen <- data.frame(iteration = j, percentage.added(data.temp$rates.filled, n.chosen))
-    results <- bind_rows(results, peradd.chosen)
-    data.plot <- rbind(
-      data.plot,
-      data.frame(iteration = j, x = peradd.chosen$start:peradd.chosen$end, y = data.temp$rates.filled[peradd.chosen$start:peradd.chosen$end], stringsAsFactors = F)
-    )
-    n <- rates.filled <- x <- y <- NULL
+    sum <- cumsumper <- totsum <- difcumsumper <- sumcum <- NULL
+    results <- results %>%
+      bind_rows(peradd.chosen) %>%
+      dplyr::mutate(sumcum = cumsum(sum), totsum = sum(data.plus$rates.filled, na.rm = T), cumsumper = sumcum / totsum) %>%
+      dplyr::mutate(difcumsumper = cumsumper - lag(cumsumper))
+    results$difcumsumper[1] <- results$cumsumper[1]
+    data.plot <- data.plot %>%
+      bind_rows(
+        data.frame(iteration = j, x = peradd.chosen$start:peradd.chosen$end, y = data.temp$rates.filled[peradd.chosen$start:peradd.chosen$end], stringsAsFactors = F) %>%
+          inner_join(results %>%
+            select(iteration, difcumsumper), by = "iteration") %>%
+          mutate(iteration.label = paste0("Iter: ", sprintf("%02d", iteration), ", Per: ", sprintf("%3.2f", 100 * difcumsumper)))
+      )
+    label <- percentage <- NULL
+    last.point <- data.plot %>%
+      group_by(iteration) %>%
+      arrange(x) %>%
+      slice(1) %>%
+      ungroup() %>%
+      bind_rows(data.plot %>%
+        group_by(iteration) %>%
+        arrange(-x) %>%
+        slice(1) %>%
+        ungroup()) %>%
+      group_by(iteration) %>%
+      arrange(y) %>%
+      slice(1) %>%
+      ungroup() %>%
+      inner_join(results %>%
+        select(iteration, percentage), by = "iteration") %>%
+      mutate(label = sprintf("%3.2f", 100 * percentage))
+
+    n <- rates.filled <- x <- y <- iteration.label <- NULL
     p2[[j]] <- ggplot() +
       geom_line(data = data.plus, aes(x = n, y = rates.filled), color = "#A0A0A0", size = 1) +
       geom_point(data = data.plus, aes(x = n, y = rates.filled), color = "#A0A0A0", size = 1.5) +
-      geom_point(data = data.plot, aes(x = x, y = y, color = factor(iteration)), size = 4) +
+      geom_point(data = data.plot, aes(x = x, y = y, color = factor(iteration.label)), size = 4) +
+      geom_point(data = last.point, aes(x = x, y = y), color = "#FFFFFF", size = 2) +
+      geom_text(data = last.point, aes(x = x, y = y, label = label), vjust = 1) +
       scale_colour_manual(values = colorRampPalette(solpalette)(j), guide = guide_legend(nrow = 3)) +
       scale_x_continuous(breaks = axis.x.ticks, limits = axis.x.range, labels = axis.x.labels) +
       scale_y_continuous(breaks = axis.y.ticks, limits = axis.y.range, labels = axis.y.labels) +
       labs(title = paste0("Iteration #", j), x = "Week", y = "Data") +
-      guides(color = guide_legend(title = "Iteration")) +
+      guides(color = guide_legend(title = paste0("Iteration (Lim: ", sprintf("%3.2f", param.2), ")"))) +
       theme_light() +
       theme(plot.title = element_text(hjust = 0.5))
-    sum <- cumsumper <- NULL
     data.temp$rates.filled[peradd.chosen$start:peradd.chosen$end] <- NA
-    results <- results %>%
-      dplyr::mutate(sumcum = cumsum(sum), cumsumper = cumsum(sum) / sum(data.plus$rates.filled, na.rm = T)) %>%
-      dplyr::mutate(difcumsumper = cumsumper - lag(cumsumper))
-    results$difcumsumper[1] <- 1
   }
   # The stopping point is determined by param.2
   max.waves.dif <- max(min.waves, min(results$iteration[results$difcumsumper < (param.2 / 100)][1] - 1, max.waves, na.rm = T), ra.rm = T)
@@ -167,7 +218,7 @@ transformseries.multiple <- function(i.data,
     guides(color = guide_legend(title = "Iteration")) +
     theme_light() +
     theme(plot.title = element_text(hjust = 0.5))
-  # Now I separate each part for the lowest vale between epidemics, but using the loess data to avoid irregular
+  # Now I separate each part for the lowest vale between epidemics, but using the smooth data to avoid irregular
   # low values between epidemics
   n.parts <- max(temp1$sunite)
   if (n.parts > 1) {
@@ -181,6 +232,7 @@ transformseries.multiple <- function(i.data,
       dplyr::slice(-1) %>%
       dplyr::select(from, to)
     cut.united <- data.frame()
+    mediann <- NULL
     for (i in 1:NROW(temp2)) {
       cut.united <- rbind(
         cut.united,
@@ -188,8 +240,9 @@ transformseries.multiple <- function(i.data,
           iteration = i,
           n = data.plus %>%
             filter(n >= temp2$from[i] & n <= temp2$to[i]) %>%
-            filter(rank(rates.loess) == 1) %>%
-            pull(n)
+            filter(rank(rates.smooth, ties.method = "min") == 1) %>%
+            summarise(mediann = quantile(n, probs = 0.50, type = 3)) %>%
+            pull(mediann)
         )
       )
     }
@@ -277,8 +330,10 @@ transformseries.multiple <- function(i.data,
     geom_vline(data = temp1, aes(xintercept = cut2), color = "#40FF40", alpha = 0.5) +
     theme_light() +
     theme(plot.title = element_text(hjust = 0.5))
-  temp2 <- axis.y.ticks[1] + diff(range(axis.y.ticks)) / 20
-  temp3 <- axis.y.ticks[1] + diff(range(axis.y.ticks)) / 15
+  # temp2 <- axis.y.ticks[1] + diff(range(axis.y.ticks)) / 20
+  # temp3 <- axis.y.ticks[1] + diff(range(axis.y.ticks)) / 15
+  temp2 <- axis.y.range[1]
+  temp3 <- axis.y.range[1]
   p4[[2]] <- ggplot() +
     geom_line(data = data.united, aes(x = n, y = rates.filled.original), color = "#A0A0A0", size = 1) +
     geom_point(data = data.united, aes(x = n, y = rates.filled.original), color = "#A0A0A0", size = 1.5, alpha = 0.75) +
@@ -287,8 +342,8 @@ transformseries.multiple <- function(i.data,
     scale_x_continuous(breaks = axis.x.ticks, limits = axis.x.range, labels = axis.x.labels) +
     scale_y_continuous(breaks = axis.y.ticks, limits = axis.y.range, labels = axis.y.labels) +
     labs(title = "Series separated and MEM epidemics", x = "Week", y = "Data") +
-    geom_segment(data = temp1, aes(x = cut1, xend = cut2, y = temp3, yend = temp3), color = "#0000FF", alpha = 0.75, size = 1, arrow = arrow(ends = "both", type = "closed", angle = "90", length = unit(5, "points"))) +
-    geom_text(data = temp1, aes(x = cut3, y = temp2, label = season), color = "#0000FF", alpha = 0.75, vjust = 1) +
+    geom_segment(data = temp1, aes(x = cut1, xend = cut2, y = temp3, yend = temp3), color = "#0000CC", alpha = 0.75, size = 1, arrow = arrow(ends = "both", type = "closed", angle = "90", length = unit(5, "points"))) +
+    geom_text(data = temp1, aes(x = cut3, y = temp2, label = season), color = "#0066CC", alpha = 0.75, vjust = 1) +
     # geom_vline(data=temp1, aes(xintercept=cut1), color="#FF0000", alpha=0.5) +
     # geom_vline(data=temp1, aes(xintercept=cut2), color="#40FF40", alpha=0.5) +
     guides(color = guide_legend(title = "Epidemic")) +
@@ -309,7 +364,7 @@ transformseries.multiple <- function(i.data,
   if (!is.na(i.output)) {
     outputdir <- file.path(getwd(), i.output)
     if (!dir.exists(outputdir)) dir.create(outputdir)
-    ggsave(paste0(i.prefix, "-1.1. Original Vs Loess.png"), p1[[1]], width = 16, height = 9, dpi = 150, path = outputdir)
+    ggsave(paste0(i.prefix, "-1.1. Original Vs Smooth.png"), p1[[1]], width = 16, height = 9, dpi = 150, path = outputdir)
     ggsave(paste0(i.prefix, "-1.2. Data to be used.png"), p1[[2]], width = 16, height = 9, dpi = 150, path = outputdir)
     # We plot each iteration to the stopping point and filter the results
     for (j in 1:max.waves.dif) ggsave(paste0(i.prefix, "-2.", j, ". Iteration ", j, ".png"), p2[[j]], width = 16, height = 9, dpi = 150, path = outputdir)
